@@ -385,6 +385,12 @@ pub trait ToLabelOrNum<'a, T : Num> {
     fn to_labelornum(self) -> LabelOrNum<T>;
 }
 
+impl<'a, T : Num> ToLabelOrNum<'a, T> for Label {
+    fn to_labelornum(self) -> LabelOrNum<T> {
+        LabelOrNum::Label(self)
+    }
+}
+
 impl<'a, T : Num> ToLabelOrNum<'a, T> for &'a Label {
     fn to_labelornum(self) -> LabelOrNum<T> {
         LabelOrNum::Label(self.clone())
@@ -432,7 +438,7 @@ struct Reference {
 /// [D16]: #method.D16
 /// [L64]: #method.L64
 /// [Label]: struct.Label.html
-pub struct Section  {
+pub struct Section {
     /// The current endianness of the writer.
     ///
     /// This sets the behavior of the D<N> appending functions.
@@ -442,7 +448,9 @@ pub struct Section  {
     /// References to `Label`s that were added to this `Section`.
     references: Vec<Reference>,
     /// A label representing the start of this `Section`.
-    start_ : Label,
+    start: Label,
+    /// A label representing the final size of this `Section`.
+    final_size: Label,
 }
 
 impl Section {
@@ -457,7 +465,8 @@ impl Section {
             endian: endian,
             contents: Cursor::new(vec!()),
             references: vec!(),
-            start_: Label::new(),
+            start: Label::new(),
+            final_size: Label::new(),
         }
     }
 
@@ -466,6 +475,13 @@ impl Section {
         self.contents.get_ref().len() as u64
     }
 
+    /// Return a `Label` that will resolve to the final size of the section.
+    ///
+    /// This label is undefined until `get_contents` is called.
+    pub fn final_size(&self) -> Label {
+        self.final_size.clone()
+    }
+
     /// Get the contents of this section as a slice of bytes.
     ///
     /// Consumes the section. If there were still undefined labels,
@@ -473,6 +489,7 @@ impl Section {
     pub fn get_contents(self) -> Option<Vec<u8>> {
         // Patch all labels into the section's contents.
         let mut section = self;
+        section.final_size.set_const(section.size());
         let references : Vec<Reference> = section.references.iter().cloned().collect();
         let mut ok = true;
         section = references.iter().cloned().fold(section, |s, r| {
@@ -499,12 +516,18 @@ impl Section {
     /// provides a single start label, for use with the Here and Mark
     /// member functions.
     pub fn start(&self) -> Label {
-        self.start_.clone()
+        self.start.clone()
     }
 
     /// A label representing the point at which the next Appended item will appear in the section, relative to start().
     pub fn here(&self) -> Label {
-        &self.start_ + self.size() as i64
+        &self.start + self.size() as i64
+    }
+
+    /// Set the value of `start()` to `value`.
+    pub fn set_start_const(self, value: u64) -> Section {
+        self.start.set_const(value);
+        self
     }
 
     /// Set `label` to Here, and return this section.
@@ -527,7 +550,8 @@ impl Section {
     /// resolved until this section is finalized.
     /// Return this section.
     pub fn append_section<S: Into<Section>>(mut self, section: S) -> Section {
-        let Section { contents, references, .. } = section.into();
+        let Section { contents, references, final_size, .. } = section.into();
+        final_size.set_const(contents.get_ref().len() as u64);
         let current = self.size();
         self.contents.write_all(&contents.into_inner()).unwrap();
         self.references.extend(references.into_iter().map(|mut r| {
@@ -923,6 +947,14 @@ fn section_append_bytes() {
                &[0, 1, 2, 3, 4, 0xf, 0xe, 0xd, 0xc, 0xb]);
 }
 
+#[test]
+fn section_final_size() {
+    let s = Section::new();
+    let size = s.final_size();
+    s.append_repeated(0, 20).get_contents().unwrap();
+    assert_eq!(size.value().unwrap(), 20);
+}
+
 #[test]
 fn section_append_section_simple() {
     assert_eq!(Section::new()
@@ -951,6 +983,18 @@ fn section_append_section_labels() {
                &[0xab, 0xcd, 0x12, 0x34, 0xef]);
 }
 
+#[test]
+fn section_append_section_final_size() {
+    let s = Section::new().D8(0xcd);
+    assert_eq!(Section::new()
+               .D8(0xab)
+               .D8(s.final_size())
+               .append_section(s)
+               .D8(0xef)
+               .get_contents().unwrap(),
+               &[0xab, 1, 0xcd, 0xef]);
+}
+
 #[test]
 fn section_append_repeated() {
     let s = Section::new();
@@ -1107,3 +1151,14 @@ fn test_simple_labels() {
 
     assert_eq!(start.offset(&end), Some(0));
 }
+
+#[test]
+fn test_set_start_const() {
+    let l = Label::new();
+    Section::new()
+        .set_start_const(0)
+        .append_repeated(0, 10)
+        .mark(&l)
+        .get_contents().unwrap();
+    assert_eq!(l.value().unwrap(), 10);
+}
